<?PHP
/*
 * Name: Model
 * Author: Michal Tatarynowicz (pies /at/ sputnik /dot/ pl)
 * Version: 0.2.0
 * Release date: 2005-04-06
 * Licence: Public Domain
 *
 * Description:
 * DBO-backed object data model, loosly based on RoR (www.rubyonrails.com).
 * Automatically selects db table name based on pluralized lowercase object class name
 * (i.e. class 'User' => table 'users'; class 'Man' => table 'men')
 * The table is required to have at least 'id auto_increment', 'created datetime', 
 * and 'modified datetime' fields
 *
 * To do:
 *   - schema-related cross-table ($has_one, $has_many, $belongs_to)
 *   - error checking with regexp, and possibly by field types (e-mail, date, address, etc.)
*/

class Model {
	// private
	var $_table = false;
	var $_id = false;
	var $_data = false;

	function Model ($id=false) {
		global $DB;

		if ($id) $this->_id = $id;
		$table_name = Inflector::pluralize(get_class($this));

		if (!in_array($table_name, $DB->tables()))
			trigger_error (sprintf(ERROR_NO_MODEL_TABLE, get_class($this), $table_name), E_USER_ERROR);
		else
			$this->_table = $table_name;
	}

	function use_table ($table_name) {
		global $DB;

		if (!in_array($table_name, $DB->tables()))
			trigger_error (sprintf(ERROR_NO_MODEL_TABLE, get_class($this), $table_name), E_USER_ERROR);
		else
			$this->_table = $table_name;
	}
	function set($one, $two=null) {
		return $this->_set_array(is_array($one)? $one: array($one=>$two));
	}
	function _set_array($data) {
		foreach ($data AS $n => $v)
			$n == 'id'? $this->_id = $v: $this->_data[$n] = $v;
	}

	// returns data from db
	// requires $this->_id
	// expects coma-separated field list, array of field names, or empty for all fields
	function read($fields=null) {
		return $this->_id? $this->find("id = '{$this->_id}'", $fields): false;
	}

	// returns a field value from db
	// requires $this->_id
	// requires a field name
	function field($name) {
		if (isset($this->_data[$name]))
			return $this->_data[$name];
		else {
			if ($this->_id && $data = $this->read($name)) {
				return isset($data[$name])? $data[$name]: false;
			}
			else {
				return false;
			}
		}
	}

	// saves $this->_data to db
	// if $this->_id is set, updates a record, else inserts a records
	function save() {
		$out = $this->_id? $this->_update(): $this->_insert();
		if ($out) $this->_data = false;
		return $out;
	}
	function _update() {
		global $DB;

		$lines = array();
		foreach ($this->_data AS $n => $v)
			$lines[] = "{$n} = '{$v}'";
		
		$update_sql = ", updated = NOW()";
			
		if (count($lines)) {
			$out = $DB->q("UPDATE {$this->_table} SET ".join(', ', $lines)."{$update_sql} WHERE id = '{$this->_id}'");

			if ($DB->got_affected())
				return true;
			else {
				return $DB->has_any($this->_table, "id = '{$this->_id}'");				
			}
		}
		else {
			return false;
		}
	}
	function _insert() {
		global $DB;

		$lines = array();
		foreach ($this->_data AS $n => $v)
			$lines[] = "{$n} = '{$v}'";

		$update_sql = ", created = NOW()";
			
		if (count($lines)) {
			if ($DB->q("INSERT INTO {$this->_table} SET ".join(', ', $lines).$update_sql)) {
				$this->_id = $DB->got_insert_id();
				return true;
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}
	}

	// deletes a record from db
	// requires $this->_id
	function del() {
		global $DB;
		if ($this->_id && $DB->q("DELETE FROM {$this->_table} WHERE id = '{$this->_id}'")) {
			$this->_id = false;
			return true;
		}
		else {
			return false;
		}
	}

	// returns true if record exists in db
	// requires $this->_id
	function exists() {
		global $DB;
		return $this->_id? $DB->has_any($this->_table, "id = '{$this->_id}'"): false;
	}

	// returns a row if found, or otherwise null
	// expects sql conditions, or empty for no conditions
	// expects coma-separated fields list, array of field names, or empty for all fields
	function find($conditions = null, $fields = null) {
		$data = $this->find_all($conditions, $fields, null, 1);
		return empty($data[0])? false: $data[0];
	}

	// returns specified fields from db records matching conditions
	// expects sql conditions, or empty for no conditions
	// expects coma-separated fields list, array of field names, or empty for all fields
	// expects sql order, or empty for default order
	// expects limit, or empty for no limit
	// expects page number for offset, or empty for no offset
	function find_all($conditions = null, $fields = null, $order = null, $limit = null, $page = null) {
		global $DB;
		
		if (is_array($fields))
			$f = $fields;
		elseif ($fields)
			$f = array($fields);
		else
			$f = array('*');

		return $DB->all(
			"SELECT "
			.join(', ', $f)
			." FROM {$this->_table}"
			.($conditions? " WHERE {$conditions}":null)
			.($order? " ORDER BY {$order}": null)
			.($limit? " LIMIT ".($page? $limit*($page-1): '0').",{$limit}": null), false, MYSQL_ASSOC);
	}
	function find_sql($sql, $debug=0) {
		global $DB;
		return $DB->all($sql, $debug);
	}
	function find_all_threaded ($conditions=null, $fields=null) {
		return $this->_do_thread($this->find_all($conditions, $fields), null);
	}
	function find_count ($conditions) {
		list($data) = $this->find_all($conditions, 'COUNT(id) AS count');
		return $data['count'];
	}
	function _do_thread($data, $root) {
		$out = array();
		
		for ($ii=0; $ii<sizeof($data); $ii++) {
			if ($data[$ii]['parent_id'] == $root) {
				$tmp = $data[$ii];
				$tmp['children'] = isset($data[$ii]['id'])? $this->_do_thread($data, $data[$ii]['id']): null;
				$out[] = $tmp;
			}
		}
		
		return $out;
	}
	function find_neighbours ($conditions, $field, $value) {
		list($prev) = $this->find_all($conditions." AND {$field} < '{$value}'", $field, "{$field} DESC", 1);
		list($next) = $this->find_all($conditions." AND {$field} > '{$value}'", $field, "{$field} ASC", 1);
		
		return array('prev'=>$prev['id'], 'next'=>$next['id']);
	}

/*
	function validate_format_of ($field_names, $rule) {
		if (!is_array($field_names))
			$field_names = array($field_names);

		foreach ($field_names AS $name)
			if (!preg_match($rule, $this->_data[$name]))
				return false;
		
		return true;
	}
*/
}

?>